pub mod common;

use common::BufferUi;
use cuddle_please::command::Command;
use tracing_test::traced_test;

use crate::common::{assert_output, get_test_data_path};

fn get_base_args<'a>() -> Vec<&'a str> {
    vec!["cuddle-please", "config", "list"]
}

#[test]
#[traced_test]
fn test_config_from_current_dir() {
    let args = get_base_args();
    let ui = &BufferUi::default();
    let current_dir = get_test_data_path("cuddle-embed");

    Command::new_from_args(Some(ui), args.into_iter())
        .execute(Some(&current_dir))
        .unwrap();

    assert_output(ui, "cuddle-config\n", "");
}

#[test]
#[traced_test]
fn test_config_from_source_dir() {
    let mut args = get_base_args();
    let ui = &BufferUi::default();
    let current_dir = get_test_data_path("cuddle-embed");
    args.push("--source");
    args.push(current_dir.to_str().unwrap());

    Command::new_from_args(Some(ui), args.into_iter())
        .execute(None)
        .unwrap();

    assert_output(ui, "cuddle-config\n", "");
}

#[test]
#[traced_test]
fn test_config_from_stdin() {
    let mut args = get_base_args();
    let ui = &BufferUi::default();
    let current_dir = get_test_data_path("cuddle-embed");
    args.push("--source");
    args.push(current_dir.to_str().unwrap());
    args.push("--config-stdin");

    Command::new_from_args_with_stdin(Some(ui), args.into_iter(), || Ok("please".into()))
        .execute(None)
        .unwrap();

    assert_output(ui, "cuddle-config\n", "");
}

#[test]
#[traced_test]
fn test_config_fails_when_not_path_is_set() {
    let args = get_base_args();
    let ui = &BufferUi::default();

    let res = Command::new_from_args(Some(ui), args.into_iter()).execute(None);

    assert!(res.is_err())
}
